c     Copyright (C) 2008-2009 Vincent Eymet
c
c     KDISTRIBUTION is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     KDISTRIBUTION is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with KDISTRIBUTION; if not, see <http://www.gnu.org/licenses/>
c
      subroutine acceleration(n,nu,k,eps1,eps2,Np,k_max,sit,z,kac,gac)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the "gac" acceleration array
c     
c     Inputs:
c       + n: size of "nu" and "k" arrays
c       + nu: array of n nu values
c       + k: array of corresponding ka values
c       + eps1,eps2: accuracy criterion (from KSPECTRUM)
c       + Np: number of points in the acceleration array
c       + k_max: maximum value of k
c       + sit: spline interpolation type (1:linear, 2:cubic)
c       + z: values of the n coefficients "z" that are needed for cubic spline interpolation
c
c     Outputs:
c       + kac: values of k for the Np acceleration points
c       + gac: values of g(k) for the Np acceleration points
c
      integer strlen
      integer n,Np,i,sit
      double precision nu(1:kmax),k(1:kmax)
      double precision eps1,eps2
      double precision z(1:kmax)
      double precision gac(0:Npmax),kac(0:Npmax),delta_g
      double precision k_max
      character*100 label
      label='acceleration'

      do i=0,Np-1
         kac(i)=k_max/dble(Np-1)*dble(i)
      enddo
      
      do i=0,Np-1
         call cumulative(n,nu,k,eps1,eps2,kac(i),sit,z,gac(i),delta_g)
      enddo
c     Debug
c      do i=0,Np-1
c         write(*,*) 'gac(',kac(i),')=',gac(i)
c      enddo
c      stop
c     Debug

c     check the final value is equal to 1
c      if (dabs(gac(Np-1)).ne.1.0D+0) then
c         write(*,*) 'gac(',Np-1,')',gac(Np-1)
c         do i=0,Np-1
c            gac(i)=gac(i)/gac(Np-1)
c         enddo ! i
c      endif
      if (dabs(gac(Np-1)).ne.1.0D+0) then
         write(*,*) 'Warning from routine ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'gac(',Np-1,')=',gac(Np-1)
         if (dabs(gac(Np-1)-1.0D+0).lt.1.0D-5) then
            do i=0,Np-1
               gac(i)=gac(i)/gac(Np-1)
            enddo               ! i
            write(*,*) 'normalization OK'
         else
            write(*,*) 'could not normalize -> error !'
         endif
      endif

      return
      end
