c     Copyright (C) 2008-2009 Vincent Eymet
c
c     KDISTRIBUTION is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     KDISTRIBUTION is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with KDISTRIBUTION; if not, see <http://www.gnu.org/licenses/>
c
      subroutine read_band(kspectrum,nl,band,nu_min,nu_max,
     &     dkdP_spectrum,dkdT_spectrum,dkdx_spectrum,Nmol,
     &     imin,n)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify the points of the provided spectrum that
c     reside within a given spectral interval
c
c     Inputs:
c       + kspectrum: nu/k(nu) points
c       + nl: number of points in the "kspectrum" spectrum
c       + band: index of the spectral narrowband interval (actually not used)
c       + nu_min: lower value of nu for the spectral interval
c       + nu_max: higher value of nu for the spectral interval
c       + dkdP_spectrum: values of d(k)/d(P) for the same values of "nu"
c       + dkdT_spectrum: values of d(k)/d(T) for the same values of "nu"
c       + dkdx_spectrum: values of d(k)/d(x) for the same values of "nu"
c       + Nmol: number of molecules in the dkdx_spectrum array
c     
c     Outputs:
c       + imin: index of the first point that reside in the [nu_min,nu_max] spectral interval
c       + n: number of points that reside in the [nu_min,nu_max] spectral interval
c
      
      integer strlen
      integer i,imin,imax,n,nl,band
      integer iminf,imaxf
      double precision nu_min,nu_max
      double precision kspectrum(1:kmax,2)
      double precision dkdP_spectrum(1:kmax)
      double precision dkdT_spectrum(1:kmax)
      double precision dkdx_spectrum(1:kmax,1:Nmol_max)
      integer Nmol
      character*100 label
      label='read_band'

c     Number of lines in the specified data file
c      call get_nlines(kfile,nl)

c     Reading the ka(nu) spectrum
c      open(11,file=kfile(1:strlen(kfile)))
      iminf=0
      do i=1,nl
         if (kspectrum(i,1).ge.nu_min) then
            imin=i
            iminf=1
            goto 123
         endif
      enddo
 123  continue
      if (iminf.eq.0) then
         write(*,*) 'Error from routine ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'Minimal frequency not found:',nu_min
c     Debug
         write(*,*) 'kspectrum(',nl,',1)=',kspectrum(nl,1)
c     Debug
         stop
      endif
      
      imaxf=0
      do i=imin,nl
         if (kspectrum(i,1).ge.nu_max) then
            imax=i-1
            imaxf=1
            goto 124
         endif
      enddo
 124  continue
      if (imaxf.eq.0) then
         write(*,*) 'Error from routine ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'Maximal frequency not found:',nu_max
         stop
      endif

      n=imax-imin+1
      if (n.le.0) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
         write(*,*) 'n=',n
         write(*,*) 'nu_min(',band,')=',nu_min
         write(*,*) 'nu_max(',band,')=',nu_max
         write(*,*) 'imin=',imin,' imax=',imax
         stop
      endif

c     This should never happen, but who knows...
      if (n.gt.kmax) then
         write(*,*) 'Error from routine ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'n=',n
         write(*,*) 'while kmax=',kmax
         stop
      endif

      return
      end
