c     Copyright (C) 2008-2009 Vincent Eymet
c
c     KDISTRIBUTION is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     KDISTRIBUTION is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c
c     You should have received a copy of the GNU General Public License
c     along with KDISTRIBUTION; if not, see <http://www.gnu.org/licenses/>
c
      subroutine read_ka(kspectrum,nl,imin,n,nu_min,nu_max,
     &     dkdP_spectrum,dkdT_spectrum,dkdx_spectrum,Nmol,
     &     nu,k,dk_dP,dk_dT,dk_dx)
      implicit none
      include 'max.inc'
c
c     Purpose: to retrieve the k-spectrum in the required spectral interval
c     
c     Inputs:
c       + kspectrum: full nu/k spectrum
c       + nl: number of points in "kspectrum"
c       + imin: index of first value that has to be taken from "kspectrum"
c       + n: number of values that have to be taken from "kspectrum"
c       + nu_min: minimum value of "nu" for the spectral interval
c       + nu_max: maximum value of "nu" for the spectral interval
c       + dkdP_spectrum: values of d(k)/d(P) for the same values of "nu"
c       + dkdT_spectrum: values of d(k)/d(T) for the same values of "nu"
c       + dkdx_spectrum: values of d(k)/d(x) for the same values of "nu"
c       + Nmol: number of molecules in the dkdx_spectrum array
c
c     Outputs:
c       + nu: values of nu for the spectral interval
c       + k: associated values of k
c       + dk_dP: values of d(k)/d(P)
c       + dk_dT: values of d(k)/d(T)
c       + dk_dx: values of d(k)/d(x)
c
      integer strlen
      integer i,imin,n,nl,k1i,kni
      double precision kspectrum(1:kmax,2)
      double precision dkdP_spectrum(1:kmax)
      double precision dkdT_spectrum(1:kmax)
      double precision dkdx_spectrum(1:kmax,1:Nmol_max)
      integer Nmol,mol
      double precision nu_min,nu_max
      double precision nu_inf,nu_sup,k_inf,k_sup
      double precision nu(1:kmax),k(1:kmax)
      double precision dk_dP(1:kmax)
      double precision dk_dT(1:kmax)
      double precision dk_dx(1:kmax,1:Nmol_max)
      double precision dkdP_inf,dkdP_sup
      double precision dkdT_inf,dkdT_sup
      double precision dkdx_inf(1:Nmol_max),dkdx_sup(1:Nmol_max)
      character*100 label
      label='read_ka'

c     Debug
c      write(*,*) 'imin=',imin,' n=',n
c      write(*,*) 'k(',imin,')=',kspectrum(imin,1),kspectrum(imin,2)
c      write(*,*) 'k(',imin+n-1,')=',kspectrum(imin+n-1,1),
c     &     kspectrum(imin+n-1,2)
c      write(*,*) 'dk_dT(',imin,')=',dkdT_spectrum(imin)
c     Debug
      k1i=0
      nu(1)=nu_min
      k(1)=kspectrum(imin,2)
      dk_dP(1)=dkdP_spectrum(imin)
      dk_dT(1)=dkdT_spectrum(imin)
      do mol=1,Nmol
         dk_dx(1,mol)=dkdx_spectrum(imin,mol)
      enddo ! mol
      if (imin.gt.1) then
         if ((nu_min.gt.kspectrum(imin-1,1))
     &        .and.(nu_min.lt.kspectrum(imin,1))) then
            k1i=1
            nu_inf=kspectrum(imin-1,1)
            k_inf=kspectrum(imin-1,2)
            dkdP_inf=dkdP_spectrum(imin-1)
            dkdT_inf=dkdT_spectrum(imin-1)
            do mol=1,Nmol
               dkdx_inf(mol)=dkdx_spectrum(imin-1,mol)
            enddo
            nu(2)=kspectrum(imin,1)
            k(2)=kspectrum(imin,2)
            dk_dP(2)=dkdP_spectrum(imin)
            dk_dT(2)=dkdT_spectrum(imin)
            do mol=1,Nmol
               dk_dx(2,mol)=dkdx_spectrum(imin,mol)
            enddo ! mol
            k(1)=k_inf+(k(2)-k_inf)/(nu(2)-nu_inf)*(nu_min-nu_inf)
            dk_dP(1)=dkdP_inf+
     &           (dk_dP(2)-dkdP_inf)
     &           /(nu(2)-nu_inf)*(nu_min-nu_inf)
            dk_dT(1)=dkdT_inf+
     &           (dk_dT(2)-dkdT_inf)
     &           /(nu(2)-nu_inf)*(nu_min-nu_inf)
            do mol=1,Nmol
               dk_dx(1,mol)=dkdx_inf(mol)+
     &              (dk_dx(2,mol)-dkdx_inf(mol))
     &              /(nu(2)-nu_inf)*(nu_min-nu_inf)
            enddo ! mol
         endif
      endif
c     Debug
c      write(*,*) 'nu(1)=',nu(1)
c      write(*,*) 'k(1)=',k(1)
c     Debug
      do i=2+k1i,n+1
         nu(i)=kspectrum(imin+i-1-k1i,1)
         k(i)=kspectrum(imin+i-1-k1i,2)
         dk_dP(i)=dkdP_spectrum(imin+i-1-k1i)
         dk_dT(i)=dkdT_spectrum(imin+i-1-k1i)
         do mol=1,Nmol
            dk_dx(i,mol)=dkdx_spectrum(imin+i-1-k1i,mol)
         enddo ! mol
      enddo
      kni=0
      if ((imin+n-1.lt.nl).or.(n.eq.0)) then
         if ((nu_max.gt.kspectrum(imin+n-1,1))
     &        .and.(nu_max.lt.kspectrum(imin+n,1))) then
            kni=1
            nu_sup=kspectrum(imin+n,1)
            k_sup=kspectrum(imin+n,2)
            dkdP_sup=dkdP_spectrum(imin+n)
            dkdT_sup=dkdT_spectrum(imin+n)
            do mol=1,Nmol
               dkdx_sup(mol)=dkdx_spectrum(imin+n,mol)
            enddo ! mol
            nu(n+2)=nu_max
            k(n+2)=k(n+1)+(k_sup-k(n+1))/(nu_sup-nu(n+1))
     &           *(nu_max-nu(n+1))
            dk_dP(n+2)=dk_dP(n+1)+
     &           (dkdP_sup-dk_dP(n+1))
     &           /(nu_sup-nu(n+1))*(nu_max-nu(n+1))
            dk_dT(n+2)=dk_dT(n+1)+
     &           (dkdT_sup-dk_dT(n+1))
     &           /(nu_sup-nu(n+1))*(nu_max-nu(n+1))
            do mol=1,Nmol
               dk_dx(n+2,mol)=dk_dx(n+1,mol)+
     &              (dkdx_sup(mol)-dk_dx(n+1,mol))
     &              /(nu_sup-nu(n+1))*(nu_max-nu(n+1))
            enddo ! mol
c     Debug
c     write(*,*) 'nu(',n+2,')=',nu(n+2)
c     write(*,*) 'k(',n+2,')=',k(n+2)
c     Debug
            n=n+1               ! because nu(n+2) is interpolated
         endif
      endif
      if (k1i.eq.1) then
         n=n+1                  ! because k(1) is interpolated
      endif

c     Debug
c      write(*,*) 'n=',n
c     Debug

c     Debug
      do i=2,n
         if (nu(i).eq.nu(i-1)) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'nu(',i-1,')=',nu(i-1)
            write(*,*) 'nu(',i,')=',nu(i)
            stop
         endif
      enddo
c     Debug

      return
      end



      subroutine read_single_kspectrum(kfile,
     &     dkdP_file,dkdT_file,dkdx_file,nu_start,nu_end,pindex,
     &     Nk,nu,k,dk_dP,dk_dT,Nmol,dk_dx)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c     
c     Purpose: to read the spectrum for a single file
c
c     Input:
c       + kfile: path and name of the file that has to be read
c       + dkdP_file: file that contains d(k)/d(P) information
c       + dkdT_file: file that contains d(k)/d(T) information
c       + dkdx_file: file that contains d(k)/d(x) information
c       + nu_start: starting wavenumber
c       + nu_end: ending wavenumber
c       + pindex: index of the process that is running this routine
c     
c     Output:
c       + Nk: number of nu/k(nu) values that have been read
c       + nu: values of nu for the spectral interval
c       + k: associated values of k
c       + dk_dP: values of d(k)/d(P)
c       + dk_dT: values of d(k)/d(T)
c       + Nmol: number of molecular species
c       + dk_dx: values of d(k)/d(x)
c

c     I/O
      character*100 kfile,dkdP_file,dkdT_file,dkdx_file
      double precision nu_start,nu_end
      integer pindex
      integer Nk
      double precision nu(1:kmax)
      double precision k(1:kmax)
      double precision dk_dP(1:kmax)
      double precision dk_dT(1:kmax)
      integer Nmol
      double precision dk_dx(1:kmax,1:Nmol_max)
c     temp
      double precision nu_tmp1,nu_tmp2
      double precision nu1,k1,raf
      double precision nua,ka
      double precision nub,kb
      double precision dkdPa,dkdPb
      double precision dkdTa,dkdTb
      double precision dkdxa(1:Nmol_max),dkdxb(1:Nmol_max)
      character*100 nlines_file
      integer n,n0,nl,nkt,nk_left
      integer i,j,ik,ik0,ik0f
      integer ios,iosP,iosT,iosx
      integer mol
      double precision k_end
      double precision dkdP_end,dkdT_end
      double precision dkdx_end(1:Nmol_max)
      character*3 str3
      integer strlen
      character*100 label
      label='subroutine read_single_kspectrum'
      
      call get_nlines(kfile,pindex,nl)

c     Debug
c      write(*,*) 'reading file:',kfile(1:strlen(kfile))
c     Debug
      open(13,file=kfile(1:strlen(kfile))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'File could not be opened:'
         write(*,*) kfile(1:strlen(kfile))
         stop
      else
c     sensitivities
         open(21,file=dkdP_file(1:strlen(dkdP_file))
     &        ,status='old',iostat=iosP)
         open(22,file=dkdT_file(1:strlen(dkdT_file))
     &        ,status='old',iostat=iosT)
         open(23,file=dkdx_file(1:strlen(dkdx_file))
     &        ,status='old',iostat=iosx)
c     sensitivities
         read(13,10) !'Pressure (atm):'
         read(13,50) !P
         read(13,10) !'Temperature (K):'
         read(13,50) !T
         read(13,10) !'Number of molecules:'
         read(13,*) Nmol
c     n is the number of lines to read in order to read the first nu/k(nu) value
         n=5*Nmol+3
c     n0 is the total number of lines at the beginning to the file
c     before the first nu/k(nu) value
         n0=n+6
c     nkt is the number of nu/k(nu) values within the file
         nkt=nl-n0
         do i=1,n
            read(13,*)
         enddo
         Nk=0
c     reading first couple nu/k(nu)
         read(13,52) nu1,raf,k1
         if (nu1.ge.nu_start) then
            Nk=Nk+1
            nu(Nk)=nu1
            k(Nk)=k1
            nk_left=nkt-1
c     sensitivities
            if (iosP.eq.0) then ! if file found
               do i=1,n0
                  read(21,*)
               enddo
               read(21,52) raf,raf,dk_dP(Nk)
            else
               dk_dP(Nk)=0.0D+0
            endif
            if (iosT.eq.0) then ! if file found
               do i=1,n0
                  read(22,*)
               enddo
               read(22,52) raf,raf,dk_dT(Nk)
            else
               dk_dT(Nk)=0.0D+0
            endif
            if (iosx.eq.0) then ! if file found
               do i=1,n0
                  read(23,*)
               enddo
               read(23,52) raf,
     &              (raf,mol=1,Nmol),
     &              (dk_dx(Nk,mol),mol=1,Nmol)
            else
               do mol=1,Nmol
                  dk_dx(Nk,mol)=0.0D+0
               enddo ! mol
            endif
c     sensitivities
         else
c     look for "nu_start"
            nu_tmp1=nu1
            ik0f=0
c     Debug
c            write(*,*) 'nkt=',nkt
c     Debug
            do ik=2,nkt
               read(13,52) nu_tmp2
c     Debug
c               write(*,*) 'nu_tmp(',ik,')=',nu_tmp(ik)
c     Debug
               if ((nu_tmp1.lt.nu_start).and.
     &              (nu_tmp2.ge.nu_start)) then
                  ik0f=1
                  ik0=ik
                  goto 111
               endif
               nu_tmp1=nu_tmp2
            enddo ! ik
 111        continue
            if (ik0f.eq.0) then
c     Debug
c               write(*,*) 'ik0 not found'
c     Debug
               goto 666
            else
               rewind(13)
               do i=1,n0+ik0-2
                  read(13,*)
               enddo
               read(13,52) nua,raf,ka ! index ik0-1
               read(13,52) nub,raf,kb ! index ik0
c     Test
               if ((nu_start.lt.nua).or.(nu_start.gt.nub)) then
                  write(*,*) 'Error from ',label(1:strlen(label)),' :'
                  write(*,*) 'nua=',nua
                  write(*,*) 'nub=',nub
                  write(*,*) 'nu_start=',nu_start
                  stop
               endif
c     Test
               Nk=Nk+1
               nu(Nk)=nu_start
c     Debug
c               write(*,*) 'nu(',Nk,')=',spectra(Nk,1)
c     Debug
               call linear_interpolation(nua,nub,ka,kb,
     &              nu(Nk),k(Nk))
c     sensitivities
               if (iosP.eq.0) then ! if file found
                  rewind(21)
                  do i=1,n0+ik0-2
                     read(21,*)
                  enddo
                  read(21,52) raf,raf,dkdPa
                  read(21,52) raf,raf,dkdPb
                  call linear_interpolation(nua,nub,dkdPa,dkdPb,
     &                 nu(Nk),dk_dP(Nk))
               else
                  dk_dP(Nk)=0.0D+0
               endif
               if (iosT.eq.0) then ! if file found
                  rewind(22)
                  do i=1,n0+ik0-2
                     read(22,*)
                  enddo
                  read(22,52) raf,raf,dkdTa
                  read(22,52) raf,raf,dkdTb
                  call linear_interpolation(nua,nub,dkdTa,dkdTb,
     &                 nu(Nk),dk_dT(Nk))
               else
                  dk_dT(Nk)=0.0D+0
               endif
               if (iosx.eq.0) then ! if file found
                  rewind(23)
                  do i=1,n0+ik0-2
                     read(23,*)
                  enddo
                  read(23,52) raf,
     &                 (raf,mol=1,Nmol),
     &                 (dkdxa(mol),mol=1,Nmol)
                  read(23,52) raf,
     &                 (raf,mol=1,Nmol),
     &                 (dkdxb(mol),mol=1,Nmol)
                  do mol=1,Nmol
                     call linear_interpolation(nua,nub,
     &                    dkdxa(mol),dkdxb(mol),
     &                    nu(Nk),dk_dx(Nk,mol))
                  enddo ! mol
               else
                  do mol=1,Nmol
                     dk_dx(Nk,mol)=0.0D+0
                  enddo ! mol
               endif
c     sensitivities
               if (nub.gt.nu_start) then
                  Nk=Nk+1
                  nu(Nk)=nub
                  k(Nk)=kb
c     Debug
c               write(*,*) 'nu(',Nk,')=',spectra(Nk,1)
c               write(*,*) nu_start,nub,(nu_start.gt.nua)
c     Debug
c     sensitivities
                  if (iosP.eq.0) then ! if file found
                     dk_dP(Nk)=dkdPb
                  else
                     dk_dP(Nk)=0.0D+0
                  endif
                  if (iosT.eq.0) then ! if file found
                     dk_dT(Nk)=dkdTb
                  else
                     dk_dT(Nk)=0.0D+0
                  endif
                  if (iosx.eq.0) then ! if file found
                     do mol=1,Nmol
                        dk_dx(Nk,mol)=dkdxb(mol)
                     enddo ! mol
                  else
                     do mol=1,Nmol
                        dk_dx(Nk,mol)=0.0D+0
                     enddo ! mol
                  endif
c     sensitivities
               endif
               nk_left=nkt-ik0
            endif               ! ik0f=0
         endif                  ! nu1 > nu_start
         
c     reading the rest of the file
         do ik=1,nk_left        ! nk_left: number of nu/k(nu) values left to read
            Nk=Nk+1
            if (Nk.gt.kmax) then
               write(*,*) 'Error from routine ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'Nk=',Nk
               write(*,*) 'is greater than kmax=',kmax
               stop
            else
               read(13,52) nu(Nk),raf,k(Nk)
c     sensitivities
               if (iosP.eq.0) then ! if file found
                  read(21,52) raf,raf,dk_dP(Nk)
               else
                  dk_dP(Nk)=0.0D+0
               endif
               if (iosT.eq.0) then ! if file found
                  read(22,52) raf,raf,dk_dT(Nk)
               else
                  dk_dT(Nk)=0.0D+0
               endif
               if (iosx.eq.0) then ! if file found
                  read(23,52) raf,
     &                 (raf,mol=1,Nmol),
     &                 (dk_dx(Nk,mol),mol=1,Nmol)
               else
                  do mol=1,Nmol
                     dk_dx(Nk,mol)=0.0D+0
                  enddo ! mol
               endif
c     sensitivities
               if (nu(Nk).gt.nu_end) then
                  call linear_interpolation(nu(Nk-1),nu(Nk),
     &                 k(Nk-1),k(Nk),nu_end,k_end)
                  nu(Nk)=nu_end
                  k(Nk)=k_end
c     sensitivities
                  if (iosP.eq.0) then ! if file found
                     call linear_interpolation(nu(Nk-1),nu(Nk),
     &                    dk_dP(Nk-1),dk_dP(Nk),nu_end,dkdP_end)
                     dk_dP(Nk)=dkdP_end
                  else
                     dk_dP(Nk)=0.0D+0
                  endif
                  if (iosT.eq.0) then ! if file found
                     call linear_interpolation(nu(Nk-1),nu(Nk),
     &                    dk_dT(Nk-1),dk_dT(Nk),nu_end,dkdT_end)
                     dk_dT(Nk)=dkdT_end
                  else
                     dk_dT(Nk)=0.0D+0
                  endif
                  if (iosx.eq.0) then ! if file found
                     do mol=1,Nmol
                        call linear_interpolation(nu(Nk-1),nu(Nk),
     &                       dk_dx(Nk-1,mol),dk_dx(Nk,mol),
     &                       nu_end,dkdx_end(mol))
                        dk_dx(Nk,mol)=dkdx_end(mol)
                     enddo ! mol
                  else
                     do mol=1,Nmol
                        dk_dx(Nk,mol)=0.0D+0
                     enddo ! mol
                  endif
c     sensitivities
                  goto 666
               endif
c     Debug
c               if (Nk.eq.2) then
c                  write(*,*) 'nu(2)=',spectra(2,1)
c               endif
c     Debug
            endif
         enddo
         close(21)
         close(22)
         close(23)
      endif                     ! ios=0

 666  continue
      close(13)

      return
      end
