c     Copyright (C) 2008-2009 Vincent Eymet
c
c     KDISTRIBUTION is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     KDISTRIBUTION is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with KDISTRIBUTION; if not, see <http://www.gnu.org/licenses/>
c
      subroutine read_spectrum(kfile,dkdP_file,dkdT_file,dkdx_file,
     &     pindex,kspectrum,nk,
     &     dkdP_spectrum,dkdT_spectrum,dkdx_spectrum,Nmol)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to read a "kspectrum" file
c
c     Inputs:
c       + kfile: path and name of the file that has to be read
c       + pindex: index of the process that is running this routine
c     
c     Outputs:
c       + kspectrum: nu/k(nu) points that have been read in the file
c       + dkdP_file: file that contains d(k)/d(P) information
c       + dkdT_file: file that contains d(k)/d(T) information
c       + dkdx_file: file that contains d(k)/d(x) information
c       + nk: number of points that have been read in the file
c       + dkdP_spectrum: values of d(k)/d(P) for the same values of "nu"
c       + dkdT_spectrum: values of d(k)/d(T) for the same values of "nu"
c       + dkdx_spectrum: values of d(k)/d(x) for the same values of "nu"
c       + Nmol: number of molecules in the dkdx_spectrum array
c

      integer i,nk,nl,pindex,ios,n0,np0
      character*100 kfile,dkdP_file,dkdT_file,dkdx_file
      double precision kspectrum(1:kmax,2),raf
      double precision dkdP_spectrum(1:kmax)
      double precision dkdT_spectrum(1:kmax)
      double precision dkdx_spectrum(1:kmax,1:Nmol_max)
      integer mol,Nmol
      integer strlen
      character*100 label
      label='read_spectrum'
 21   format(a20)
      
c     Number of lines in the specified data file
      call get_nlines(kfile,pindex,nl)
c     Debug
c      write(*,*) 'file=',kfile(1:strlen(kfile)),' nl=',nl
c     Debug

      open(11,file=kfile(1:strlen(kfile)),status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         write(*,*) 'Error from routine ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'File not found:'
         write(*,*) kfile(1:strlen(kfile))
         stop
      else
         do i=1,5
            read(11,*)
         enddo
         read(11,*) Nmol
         n0=5*Nmol+3
         np0=n0+6
         nk=nl-np0
         if (nk.gt.kmax) then
            write(*,*) 'Error from routine ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'Number of points in the spectrum:',nl
            write(*,*) 'Maximum allowed number of points:',kmax
            write(*,*) 'Increase variable "kmax"'//
     &           'in file "includes/max.inc"'
            write(*,*) 'and then recompile using "make clean all"'
            stop
         endif
         do i=1,n0
            read(11,*)
         enddo
         do i=1,nk
            read(11,52) kspectrum(i,1),raf,kspectrum(i,2) ! nu,sigma,k
c     Debug
c            write(*,*) i,kspectrum(i,1),kspectrum(i,2)
c     Debug
         enddo
      endif
      close(11)

c     ----------------------------------------------------------------------
c     sensitivities
      open(12,file=dkdP_file(1:strlen(dkdP_file)),
     &     status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         do i=1,nk
            dkdP_spectrum(i)=0.0D+0
         enddo ! i
      else
         do i=1,np0
            read(12,*)
         enddo
         do i=1,nk
            read(12,52) raf,raf,dkdP_spectrum(i)
         enddo
      endif
      close(12)

      open(13,file=dkdT_file(1:strlen(dkdT_file)),
     &     status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         do i=1,nk
            dkdT_spectrum(i)=0.0D+0
         enddo ! i
      else
         do i=1,np0
            read(13,*)
         enddo
         do i=1,nk
            read(13,52) raf,raf,dkdT_spectrum(i)
         enddo
      endif
      close(13)
c     Debug
c      open(10,file='./results/inconsistencies.txt')
c      do i=1,nl
c         if ((kspectrum(i,2)-dkdT_spectrum(i)).lt.0.0D+0) then
c            write(10,*) kspectrum(i,1),kspectrum(i,2),dkdT_spectrum(i)
c         endif
c      enddo
c      close(10)
c     Debug

      open(14,file=dkdx_file(1:strlen(dkdx_file)),
     &     status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         Nmol=1
         do i=1,nk
            dkdx_spectrum(i,Nmol)=0.0D+0
         enddo ! i
      else
         do i=1,np0
            read(14,*)
         enddo
         do i=1,nk
            read(14,52) raf,
     &           (raf,mol=1,Nmol),
     &           (dkdx_spectrum(i,mol),mol=1,Nmol)
         enddo
      endif
      close(14)
c     ----------------------------------------------------------------------

      return
      end
