      subroutine other_spectrum(n,k,dk_dP,dk_dT,dk_dx,Nmol,
     &     i,pres,temp,x,
     &     kP,kT,kx,dP,dT,dx)
      implicit none
      include 'max.inc'
c
c     Purpose: to produce values of kP, kT, kx and associated
c     increments dP, dT and dx
c
c     Input:
c       + n: number of values in arrays "k", "dk_dXXX"
c       + k: values of k
c       + dk_dP: values of d(k)/d(P)
c       + dk_dT: values of d(k)/d(T)
c       + dk_dx: values of d(k)/d(x)
c       + Nmol: number of molecules in the dk_dx array
c       + i: index of pressure level
c       + pres: array of total pressure
c       + temp: array of temperature
c       + x: array of concentrations
c
c     Output:
c       + kP: values of k+d(k)/d(P)*dP
c       + kT: values of k+d(k)/d(T)*dT
c       + kx: values of k+d(k)/d(x)*dx
c       + dP: increment of P
c       + dT: increment of T
c       + dx: increment of x
c

c     I/O
      integer n
      double precision k(1:kmax)
      double precision dk_dP(1:kmax)
      double precision dk_dT(1:kmax)
      double precision dk_dx(1:kmax,1:Nmol_max)
      integer Nmol,i
      double precision pres(1:Nmax)
      double precision temp(1:Nmax)
      double precision x(1:Nmax,1:Nmol_max)
      double precision kP(1:kmax)
      double precision kT(1:kmax)
      double precision kx(1:kmax,1:Nmol_max)
      double precision dP,dT,dx(1:Nmol_max)
c     temp
      integer mol,ik,ok
      integer strlen
      character*100 label
      label='subroutine other_spectrum'
      
      dP=pres(i)/1.0D+3 ! atm
c     Debug
c      write(*,*) 'pres(',i,')=',pres(i),' dP=',dP
c     Debug
      dT=1.0D+0 ! K
      do mol=1,Nmol
         dx(mol)=x(i,mol)/1.0D+2
      enddo

      ok=0
      do while (ok.eq.0)
         ok=1
         do ik=1,n
            kP(ik)=k(ik)+dk_dP(ik)*dP
            if (kP(ik).lt.0.0D+0) then
c               write(*,*) 'Warning from ',label(1:strlen(label)),' :'
c               write(*,*) 'k=',k(ik)
c               write(*,*) 'dkdP=',dk_dP(ik)
c               write(*,*) 'dP=',dP
c               write(*,*) 'dP=',kP(ik)
               dP=dP/2.0D+0
c               write(*,*) 'Now trying with dP=',dP
               ok=0
               goto 111
            endif
         enddo ! ik
 111     continue
      enddo


      ok=0
      do while (ok.eq.0)
         ok=1
         do ik=1,n
            kT(ik)=k(ik)+dk_dT(ik)*dT
            if (kT(ik).lt.0.0D+0) then
c               write(*,*) 'Warning from ',label(1:strlen(label)),' :'
c               write(*,*) 'k=',k(ik)
c               write(*,*) 'dkdT=',dk_dT(ik)
c               write(*,*) 'dT=',dT
c               write(*,*) 'dT=',kT(ik)
               dT=dT/2.0D+0
c               write(*,*) 'Now trying with dT=',dT
               ok=0
               goto 112
            endif
         enddo ! ik
 112     continue
      enddo

      do mol=1,Nmol
         ok=0
         do while (ok.eq.0)
            ok=1
            do ik=1,n
               kx(ik,mol)=k(ik)+dk_dx(ik,mol)*dx(mol)
               if (kx(ik,mol).lt.0.0D+0) then
c                  write(*,*) 'Warning from ',label(1:strlen(label)),' :'
c                  write(*,*) 'k=',k(ik)
c                  write(*,*) 'dkdx=',dk_dx(ik,mol)
c                  write(*,*) 'dx=',dx(mol)
c                  write(*,*) 'dx=',kx(ik,mol)
                  dx(mol)=dx(mol)/2.0D+0
c                  write(*,*) 'Now trying with dx(',mol,')=',dx(mol)
                  ok=0
                  goto 113
               endif
            enddo               ! ik
 113        continue
         enddo
      enddo                     ! mol

      return
      end

