c     Copyright (C) 2008-2009 Vincent Eymet
c
c     KDISTRIBUTION is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     KDISTRIBUTION is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with KDISTRIBUTION; if not, see <http://www.gnu.org/licenses/>
c
      subroutine write_kdist_results(i_min,i_max,Nb,Nq,nu_min,nu_max,
     &     w,kdist,delta_k,tr_exact,tr_quad,
     &     kdistP,kdistT,kdistx,Nmol,dP,dT,dx)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to write k-distribution output results
c     
c     Inputs:
c       + i_min,i_max: min and max interface indexes
c       + Nb: number of narrowband spectral intervals
c       + Nq: quadrature order
c       + nu_min, nu_max: minimum and maximum values of "nu"
c       + w: array of "Nq" quadrature weights
c       + kdist: output k-distribution data set
c       + delta_k: uncertainties over k-distribution results
c       + tr_exact: mean transmittivities computed from the initial discretized k-spectrum
c       + tr_quad: mean transmittivities computed from the output k-distribution data set
c       + kdistP: output k-distribution data set for P+dP
c       + kdistT: output k-distribution data set for T+dT
c       + kdistx: output k-distribution data set for x+dx
c       + Nmol: number of molecular species
c       + dP: pressure increments
c       + dT: temperature increments
c       + dx: concentration increments
c
      integer strlen
      integer Nb,Nq,band,quad,i_min,i_max,i
      double precision nu_min(1:Nbmx),nu_max(1:Nbmx)
      double precision w(Nqmx)
      double precision kdist(1:Nmax,1:Nbmx,1:Nqmx)
      double precision delta_k(1:Nmax,1:Nbmx,1:Nqmx)
      double precision tr_exact(1:Nmax,1:Nbmx)
      double precision tr_quad(1:Nmax,1:Nbmx)
      double precision kdistP(1:Nmax,1:Nbmx,1:Nqmx)
      double precision kdistT(1:Nmax,1:Nbmx,1:Nqmx)
      double precision kdistx(1:Nmax,1:Nbmx,1:Nqmx,1:Nmol_max)
      integer Nmol,mol
      double precision dP(1:Nmax,1:Nbmx)
      double precision dT(1:Nmax,1:Nbmx)
      double precision dx(1:Nmax,1:Nbmx,1:Nmol_max)
      double precision dkdistdP(1:Nmax,1:Nbmx,1:Nqmx)
      double precision dkdistdT(1:Nmax,1:Nbmx,1:Nqmx)
      double precision dkdistdx(1:Nmax,1:Nbmx,1:Nqmx,1:Nmol_max)
      double precision err(1:Nmax,1:Nbmx)
      character*100 label
      label='write_kdist_results'
c     Debug
c      double precision ka_moy(1:Nmax,1:Nbmx)
c     Debug

      do i=i_min,i_max
         do band=1,Nb
            if (tr_exact(i,band).gt.1.0D-10) then
               err(i,band)=dabs(tr_exact(i,band)-tr_quad(i,band))
     &              /dabs(tr_exact(i,band))
            else
               err(i,band)=-1.0D+0
            endif
         enddo
      enddo
      

      open(12,file='results/kdist.txt')
      write(12,*) 'Number of spectral intervals:',Nb
      write(12,*) 'Quadrature order:',Nq
      write(12,*) 'Minimum interface index:',i_min
      write(12,*) 'Maximum interface index:',i_max
      write(12,*)
      write(12,*) 'Quadrature weights w(quad):'
      do quad=1,Nq
         write(12,*) w(quad)
      enddo
      write(12,*)
      write(12,*) 'nu_min / nu_max / k(quad),quad=1,Nq / err'
      do i=i_min,i_max
         write(12,*) 'Interface:',i
         do band=1,Nb
            write(12,50) nu_min(band),nu_max(band),
     &           (kdist(i,band,quad),quad=1,Nq),
     &           err(i,band)
         enddo                  ! band
         write(12,*)
      enddo
      close(12)

c     ----------------------------------------------------------------------
c     sensitivities:
c     + to pressure
      do i=i_min,i_max
         do band=1,Nb
            do quad=1,Nq
               dkdistdP(i,band,quad)=
     &              (kdistP(i,band,quad)-kdist(i,band,quad))
     &              /dP(i,band)
            enddo ! quad
         enddo ! band
      enddo ! i

      open(13,file='results/dkdist_dP.txt')
      write(13,*) 'Number of spectral intervals:',Nb
      write(13,*) 'Quadrature order:',Nq
      write(13,*) 'Minimum interface index:',i_min
      write(13,*) 'Maximum interface index:',i_max
      write(13,*)
      write(13,*) 'Quadrature weights w(quad):'
      do quad=1,Nq
         write(13,*) w(quad)
      enddo
      write(13,*)
      write(13,*) 'nu_min / nu_max / k(quad),quad=1,Nq / err'
      do i=i_min,i_max
         write(13,*) 'Interface:',i
         do band=1,Nb
            write(13,50) nu_min(band),nu_max(band),
     &           (dkdistdP(i,band,quad),quad=1,Nq)
         enddo                  ! band
         write(13,*)
      enddo
      close(13)
c     + to temperature
      do i=i_min,i_max
         do band=1,Nb
            do quad=1,Nq
               dkdistdT(i,band,quad)=
     &              (kdistT(i,band,quad)-kdist(i,band,quad))
     &              /dT(i,band)
            enddo ! quad
         enddo ! band
      enddo ! i

      open(14,file='results/dkdist_dT.txt')
      write(14,*) 'Number of spectral intervals:',Nb
      write(14,*) 'Quadrature order:',Nq
      write(14,*) 'Minimum interface index:',i_min
      write(14,*) 'Maximum interface index:',i_max
      write(14,*)
      write(14,*) 'Quadrature weights w(quad):'
      do quad=1,Nq
         write(14,*) w(quad)
      enddo
      write(14,*)
      write(14,*) 'nu_min / nu_max / k(quad),quad=1,Nq / err'
      do i=i_min,i_max
         write(14,*) 'Interface:',i
         do band=1,Nb
            write(14,50) nu_min(band),nu_max(band),
     &           (dkdistdT(i,band,quad),quad=1,Nq)
         enddo                  ! band
         write(14,*)
      enddo
      close(14)
c     + to concentrations
      do i=i_min,i_max
         do band=1,Nb
            do quad=1,Nq
               do mol=1,Nmol
                  dkdistdx(i,band,quad,mol)=
     &                 (kdistx(i,band,quad,mol)-kdist(i,band,quad))
     &                 /dx(i,band,mol)
               enddo ! mol
            enddo ! quad
         enddo ! band
      enddo ! i

      open(15,file='results/dkdist_dx.txt')
      write(15,*) 'Number of spectral intervals:',Nb
      write(15,*) 'Number of molecules:',Nmol
      write(15,*) 'Quadrature order:',Nq
      write(15,*) 'Minimum interface index:',i_min
      write(15,*) 'Maximum interface index:',i_max
      write(15,*)
      write(15,*) 'Quadrature weights w(quad):'
      do quad=1,Nq
         write(15,*) w(quad)
      enddo
      write(15,*)
      write(15,*) 'nu_min / nu_max / k(quad),quad=1,Nq / err'
      do mol=1,Nmol
         write(15,*) 'molecule number:',mol
         do i=i_min,i_max
            write(15,*) 'Interface:',i
            do band=1,Nb
               write(15,50) nu_min(band),nu_max(band),
     &              (dkdistdx(i,band,quad,mol),quad=1,Nq)
            enddo               ! band
            write(15,*)
         enddo                  ! i
      enddo                     !mol
      close(15)
c     ----------------------------------------------------------------------
      
c     Debug
c      do i=i_min,i_max
c         do band=1,Nb
c            ka_moy(i,band)=0.0D+0
c            do quad=1,Nq
c               ka_moy(i,band)=ka_moy(i,band)
c     &              +kdist(i,band,quad)**w(quad)
c            enddo               ! quad
c         enddo                  ! band
c      enddo                     ! i
c      open(10,file='./results/ka_moy.txt')
c      do i=i_min,i_max
c         write(10,*) 'level:',i
c         write(10,50) (ka_moy(i,band),band=1,Nb)
c      enddo
c      close(10)
c     Debug



      open(14,file='results/delta_kdist.txt')
      write(14,*) 'Number of spectral intervals:',Nb
      write(14,*) 'Quadrature order:',Nq
      write(14,*) 'Minimum interface index:',i_min
      write(14,*) 'Maximum interface index:',i_max
      write(14,*)
      write(14,*) 'Quadrature weights w(quad):'
      do quad=1,Nq
         write(14,*) w(quad)
      enddo
      write(14,*)
      write(14,*) 'nu_min / nu_max / delta(k)(quad),quad=1,Nq'
      do i=i_min,i_max
         write(14,*) 'Interface:',i
         do band=1,Nb
            write(14,50) nu_min(band),nu_max(band),
     &           (delta_k(i,band,quad),quad=1,Nq)
         enddo                  ! band
         write(14,*)
      enddo
      close(14)

      return
      end
